//-----------------------------------------------------------------------------
// name: mo_compass.mm
// desc: MoPhO API for compass
//
// authors: Nick Bryan
//          Jorge Herrera
//          Jieun Oh
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#include "mo_compass.h"


@implementation CompassDelegate

@synthesize locationManager;

- (void)locationManager:(CLLocationManager *)manager didUpdateHeading:(CLHeading *)heading
{
    // update the compass server data
    MoCompass::instance()->update( heading );
}

@end


//-----------------------------------------------------------------------------
// name: instance()
// desc: returns the singelton instance
//-----------------------------------------------------------------------------
MoCompass * MoCompass::ourInstance = NULL;
MoCompass * MoCompass::instance()
{
    if(!ourInstance)
    {
        ourInstance = new MoCompass();
        ourInstance->setup();
    }

    return ourInstance;	
}


//-----------------------------------------------------------------------------
// name: MoCompass()
// desc: the private constructor that is called only if it's the first and only
//       instance of the MoAccel Class
//-----------------------------------------------------------------------------
MoCompass::MoCompass()
{
    m_trueOffset = 0;
    m_magneticOffset = 0;

    m_magneticHeading = 0.0;
    m_trueHeading = 0.0;
    m_accuracy = 0.0;
}


//-----------------------------------------------------------------------------
// name: ~MoCompass()
// desc: the private constructor that is called only if it's the first and only
//       instance of the MoAccel Class
//-----------------------------------------------------------------------------
MoCompass::~MoCompass()
{ }


void MoCompass::setup()
{
    // allocate a location Delegate object
    compassDelegate = [CompassDelegate alloc];

    compassDelegate.locationManager = [[[CLLocationManager alloc] init] autorelease];

    // check if the hardware has a compass
    if (compassDelegate.locationManager.headingAvailable == NO) {
        // No compass is available. This application cannot function without a compass, 
        // so a dialog will be displayed and no magnetic data will be measured.
        compassDelegate.locationManager = nil;
        UIAlertView *noCompassAlert = [[UIAlertView alloc] initWithTitle:@"No Compass!" 
                                                                 message:@"This device does not have the ability to measure magnetic fields." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [noCompassAlert show];
        [noCompassAlert release];
    } else {
        // heading service configuration
        compassDelegate.locationManager.headingFilter = kCLHeadingFilterNone;

        // this is default anyway, but just to make sure
        compassDelegate.locationManager.desiredAccuracy = kCLLocationAccuracyBest;

        // setup delegate callbacks
        compassDelegate.locationManager.delegate = compassDelegate;

        // start the compass
        [compassDelegate.locationManager startUpdatingHeading];
    }
}


//-----------------------------------------------------------------------------
// name: update()
// desc: update the internal accelerometer data and process any callbacks
//-----------------------------------------------------------------------------
void MoCompass::update( CLHeading * heading )
{
    // update the current heading
    m_heading = heading;

    m_magneticHeading = [heading magneticHeading];
    m_trueHeading = [heading trueHeading];
    m_accuracy = [heading headingAccuracy];

	// process all callbacks
    for( int i=0; i < m_clients.size(); i++ ) // TODO: Change to using iterators
        (m_clients[i])( m_heading, m_clientData[i]);  
}


//-----------------------------------------------------------------------------
// name: getMagneticHeading()
// desc: gets the magnetic heading of the compass
//-----------------------------------------------------------------------------
double MoCompass::getMagneticHeading() const
{
    return m_magneticHeading;
}


//-----------------------------------------------------------------------------
// name: getTrueHeading()
// desc: gets the true heading of the compass
//-----------------------------------------------------------------------------
double MoCompass::getTrueHeading() const
{
    return m_trueHeading;
}


//-----------------------------------------------------------------------------
// name: getAccuracy()
// desc: gets the accuracy in degress of the current heading value
//-----------------------------------------------------------------------------
double MoCompass::getAccuracy() const
{
    return m_accuracy;
}


/*//-----------------------------------------------------------------------------
// name: getTimestamp()
// desc: gets the timestamp of the current heading value
//-----------------------------------------------------------------------------
NSDate * MoCompass::getTimestamp() const
{
	return [m_heading timestamp];
}*/


//-----------------------------------------------------------------------------
// name: setOffset()
// desc: stores an offset using the current magnetic heading of the compass to be used later
//-----------------------------------------------------------------------------
void MoCompass::setOffset()
{
    NSLog(@"Seeing Offset\n");
    m_magneticOffset = getMagneticHeading();
    m_trueOffset = getTrueHeading();
}


//-----------------------------------------------------------------------------
// name: clearOffset()
// desc: stores an offset using the current magnetic heading of the compass to be used later
//-----------------------------------------------------------------------------
void MoCompass::clearOffset()
{
    m_magneticOffset = 0;
    m_trueOffset = 0;
}


double MoCompass::getMagneticOffset()
{
    return m_magneticOffset;
}

double MoCompass::getTrueOffset()
{
    return m_trueOffset;
}


//-----------------------------------------------------------------------------
// name:  add
// desc:  registers a callback to be invoked on subsequent updates       
//-----------------------------------------------------------------------------
void  MoCompass::add(const MoCompassCallback & callback, void * data )
{
    // NSLog(@"Adding MoCompassCallback\n");
    m_clients.push_back( callback );
    m_clientData.push_back(data);
}


//-----------------------------------------------------------------------------
// name:  add
// desc:  unregisters a callback to be invoked on subsequent updates       
//-----------------------------------------------------------------------------
void  MoCompass::remove(const MoCompassCallback & callback )
{
    // NSLog(@"Removing MoCompassCallback\n");
    // find the callback and remove
    // TODO: Change to using iterators
    for( int i=0; i < m_clients.size(); i++ )
    {
        if(m_clients[i]==callback)
        {
            m_clients.erase(m_clients.begin()+i);
            m_clientData.erase(m_clientData.begin()+i);
        }
    }

    // TODO: would find work???
}
