//-----------------------------------------------------------------------------
// name: mo_compass.mm
// desc: MoPhO API for compass
//
// authors: Nick Bryan
//          Jorge Herrera
//          Jieun Oh
//          Ge Wang
//
//    date: Fall 2009
//    version: 0.1
//
// Stanford Mobile Phone Orchestra
//     http://mopho.stanford.edu/
//-----------------------------------------------------------------------------
#include "mo_location.h"


@implementation LocationDelegate

@synthesize locationManager;

- (void)locationManager:(CLLocationManager *)manager 
    didUpdateToLocation:(CLLocation *)newLocation 
           fromLocation:(CLLocation *)oldLocation
{
    // Update the Location data
    MoLocation::instance()->update(newLocation, oldLocation);
}


- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error
{
    // the location "unknown" error simply means the manager is currently unable to get the location.
    // if ([error code] != kCLErrorLocationUnknown) {
    //    [self stopUpdatingLocation:NSLocalizedString(@"Error", @"Error")];
    // }
}

@end


//-----------------------------------------------------------------------------
// name: instance()
// desc: returns the singelton instance
//-----------------------------------------------------------------------------
MoLocation * MoLocation::ourInstance = NULL;
MoLocation * MoLocation::instance()
{
    if(!ourInstance)
    {
        ourInstance = new MoLocation();
        ourInstance->setup();
    }

    return ourInstance;	
}


//-----------------------------------------------------------------------------
// name: MoCompass()
// desc: the private constructor that is called only if it's the first and only
//       instance of the MoAccel Class
//-----------------------------------------------------------------------------
MoLocation::MoLocation()
{ }


void MoLocation::setup()
{
    // allocate a location Delegate object
    locationDelegate = [LocationDelegate alloc];

    locationDelegate.locationManager = [[[CLLocationManager alloc] init] autorelease];

    // check if the hardware has a compass
    if (locationDelegate.locationManager.headingAvailable == NO) {
        // no compass is available. This application cannot function without a compass, 
        // so a dialog will be displayed and no magnetic data will be measured.
        locationDelegate.locationManager = nil;
        UIAlertView *noCompassAlert = [[UIAlertView alloc] initWithTitle:@"No Compass!" message:@"This device does not have the ability to measure magnetic fields." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [noCompassAlert show];
        [noCompassAlert release];
    }
    else {
        // heading service configuration
        locationDelegate.locationManager.distanceFilter = kCLDistanceFilterNone;

        //This is default anyway, but just to make sure
        locationDelegate.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
		
        // setup delegate callbacks
        locationDelegate.locationManager.delegate = locationDelegate;

        // start the compass
        [locationDelegate.locationManager startUpdatingLocation];
    }
}


//-----------------------------------------------------------------------------
// name: update()
// desc: update the internal accelerometer data and process any callbacks
//-----------------------------------------------------------------------------
void MoLocation::update( CLLocation * newLoc, CLLocation * oldLoc )
{
    m_newLocation = (CLLocation *)newLoc;
    m_oldLocation = (CLLocation *)oldLoc;

    // process all callbacks
    for( int i=0; i < m_clients.size(); i++ ) // TODO: change to using iterators
        (m_clients[i])( m_newLocation, m_oldLocation, m_clientData[i]);  
}

CLLocation * MoLocation::getLocation()
{
    return m_newLocation;
}

CLLocation * MoLocation::getOldLocation()
{
    return m_oldLocation;
}



//-----------------------------------------------------------------------------
// name: add
// desc: registers a callback to be invoked on subsequent updates       
//-----------------------------------------------------------------------------
void MoLocation::add(const MoLocationCallback & callback, void * data )
{
    // NSLog(@"Adding MoLocationCallback\n");
    m_clients.push_back( callback );
    m_clientData.push_back(data);
}


//-----------------------------------------------------------------------------
// name: add
// desc: unregisters a callback to be invoked on subsequent updates       
//-----------------------------------------------------------------------------
void MoLocation::remove(const MoLocationCallback & callback )
{
    // NSLog(@"Removing MoLocationCallback\n");
    // find the callback and remove
    // TODO: Change to using iterators
    for(int i=0; i < m_clients.size(); i++) 	
    {
        if(m_clients[i]==callback)
        {
            m_clients.erase(m_clients.begin()+i);
            m_clientData.erase(m_clientData.begin()+i);
        }
    }
	
    //TODO: would find work???
}
